#include <iostream>
#include "datagrid.h"
#include <cmath>

using namespace std;

#define DELTA 0.0001

datagrid::datagrid(){
  nx=1;
  ny=1;
  e1.resize(1);
  e2.resize(1);
  w.resize(1);
}
datagrid::datagrid(int nx_new) {
  nx=nx_new;
  ny=nx_new;
  e1.resize(nx*ny);
  e2.resize(nx*ny);
  w.resize(nx*ny);
}
datagrid::datagrid(int nx_new, int n, vector<double> x, vector<double> y, vector<double> e1_arr,vector<double> e2_arr, vector<double> w_arr) {
  nx=nx_new;
  ny=nx_new;
  e1.resize(nx*ny);
  e2.resize(nx*ny);
  w.resize(nx*ny);
  for (int i=0; i < nx*ny; i++) {
    e1[i]=0;
    e2[i]=0;
    w[i]=0;
  }
  for (int idx=0; idx < n ; idx++) {
    //int i=int(x[idx]*nx);
    //int j=int(y[idx]*nx);			
    int i0=int(x[idx]*nx+0.5)-1;
    int j0=int(y[idx]*ny+0.5)-1;
    double wx=x[idx]*nx-0.5-i0; // 0 means full weighting in i0.
    double wy=y[idx]*ny-0.5-j0;
    for (int i=i0; i < i0+2; i++) {
      for (int j=j0 ; j < j0+2; j++) {
	if ((i >= 0)&&(i < nx)&&(j >= 0)&&(j < ny)) {
	  double w_loc=abs(((i-i0)-wx)*((j-j0)-wy));
	  int ibin=i*ny+j;
	  e1[ibin]+=w_loc*w_arr[idx]*e1_arr[idx];
	  e2[ibin]+=w_loc*w_arr[idx]*e2_arr[idx];
	  w[ibin]+=w_loc*w_arr[idx];
	}
      }
    }
  }
  for (int ibin=0; ibin < nx*ny; ibin++) {
    if (w[ibin] > DELTA) {
      e1[ibin]=e1[ibin]/w[ibin];
      e2[ibin]=e2[ibin]/w[ibin];
    }
  }
}

double datagrid::get_data(string name, int i, int j) {
  // 0 returns w
  // 1 returns e1
  // 2 returns e2
  if ((i >= 0)&&(i < nx)&&(j>=0) && (j < nx)) {
    int ibin=j+i*nx;
    if (name.find("W") != string::npos) {
      return(w[ibin]);
    }
    else if (name.find("E1") != string::npos) {
      return(e1[ibin]);
    } 
    else if (name.find("E2") != string::npos) {
      return(e2[ibin]);
    }
  }
  return 0;
}

vector<double> datagrid::get_data(string name){
  if (name == "E1") return e1;
  if (name == "E2") return e2;
  if (name == "W") return w;
}
